@@ -91,7 +91,7 @@ class GeneratorTestCase(base.BaseTestCase):
             deprecated_for_removal=True,
             deprecated_reason='This was supposed to work but it really, '
                               'really did not. Always buy house insurance.',
-            help='Turn off stove'),
+            help='DEPRECATED: Turn off stove'),
         'deprecated_group': cfg.StrOpt('bar',
                                        deprecated_group='group1',
                                        deprecated_name='foobar',
@@ -142,6 +142,9 @@ class GeneratorTestCase(base.BaseTestCase):
         'port_opt': cfg.PortOpt('port_opt',
                                 default=80,
                                 help='a port'),
+        'hostname_opt': cfg.HostnameOpt('hostname_opt',
+                                        default='compute01.nova.site1',
+                                        help='a hostname'),
         'multi_opt': cfg.MultiStrOpt('multi_opt',
                                      default=['1', '2', '3'],
                                      help='multiple strings'),
@@ -424,7 +427,7 @@ class GeneratorTestCase(base.BaseTestCase):
 # From test
 #
 
-# deprecated for removal (string value)
+# DEPRECATED: deprecated for removal (string value)
 # This option is deprecated for removal.
 # Its value may be silently ignored in the future.
 #bar = <None>
@@ -442,7 +445,7 @@ class GeneratorTestCase(base.BaseTestCase):
 # From test
 #
 
-# Turn off stove (boolean value)
+# DEPRECATED: Turn off stove (boolean value)
 # This option is deprecated for removal.
 # Its value may be silently ignored in the future.
 # Reason: This was supposed to work but it really, really did not.
@@ -604,6 +607,17 @@ class GeneratorTestCase(base.BaseTestCase):
 # Minimum value: 0
 # Maximum value: 65535
 #port_opt = 80
+''')),
+        ('hostname_opt',
+         dict(opts=[('test', [(None, [opts['hostname_opt']])])],
+              expected='''[DEFAULT]
+
+#
+# From test
+#
+
+# a hostname (hostname value)
+#hostname_opt = compute01.nova.site1
 ''')),
         ('multi_opt',
          dict(opts=[('test', [(None, [opts['multi_opt']])])],
@@ -696,6 +710,78 @@ class GeneratorTestCase(base.BaseTestCase):
 
 # string with bad default (string value)
 #string_type_with_bad_default = 4096
+''')),
+         ('str_opt_str_group',
+         dict(opts=[('test', [('foo',
+                               [opts['str_opt']]),
+                              (groups['foo'],
+                               [opts['int_opt']])]),
+                    ('foo', [('foo',
+                               [opts['bool_opt']])])],
+              expected='''[DEFAULT]
+
+
+[foo]
+# foo help
+
+#
+# From foo
+#
+
+# a boolean (boolean value)
+#bool_opt = false
+
+#
+# From test
+#
+
+# a string (string value)
+#str_opt = foo bar
+
+#
+# From test
+#
+
+# an integer (integer value)
+# Minimum value: 1
+# Maximum value: 20
+#int_opt = 10
+''')),
+         ('opt_str_opt_group',
+         dict(opts=[('test', [(groups['foo'],
+                               [opts['int_opt']]),
+                              ('foo',
+                               [opts['str_opt']])]),
+                    ('foo', [(groups['foo'],
+                              [opts['bool_opt']])])],
+              expected='''[DEFAULT]
+
+
+[foo]
+# foo help
+
+#
+# From foo
+#
+
+# a boolean (boolean value)
+#bool_opt = false
+
+#
+# From test
+#
+
+# an integer (integer value)
+# Minimum value: 1
+# Maximum value: 20
+#int_opt = 10
+
+#
+# From test
+#
+
+# a string (string value)
+#str_opt = foo bar
 ''')),
     ]
 
@@ -768,8 +854,9 @@ class GeneratorTestCase(base.BaseTestCase):
         if self.stdout:
             self.assertEqual(self.expected, stdout.getvalue())
         else:
-            content = open(output_file).read()
-            self.assertEqual(self.expected, content)
+            with open(output_file, 'r') as f:
+                actual = f.read()
+            self.assertEqual(self.expected, actual)
 
         log_warning = getattr(self, 'log_warning', None)
         if log_warning is not None:
@@ -855,19 +942,10 @@ class GeneratorAdditionalTestCase(base.BaseTestCase):
             cfg.BoolOpt('bool_opt', help='a boolean'),
             cfg.IntOpt('int_opt', help='an integer')]
 
-    def test_get_group_name(self):
-        name = "group1"
-        item = [name]
-        self.assertEqual(name, generator._get_group_name(item))
-
-    def test_get_group_name_as_optgroup(self):
-        name = "group2"
-        item = [cfg.OptGroup(name)]
-        self.assertEqual(name, generator._get_group_name(item))
-
     def test_get_groups_empty_ns(self):
         groups = generator._get_groups([])
-        self.assertEqual({'DEFAULT': []}, groups)
+        self.assertEqual({'DEFAULT': {'object': None, 'namespaces': []}},
+                         groups)
 
     def test_get_groups_single_ns(self):
         config = [("namespace1", [
@@ -893,14 +971,14 @@ class GeneratorAdditionalTestCase(base.BaseTestCase):
         groups = generator._get_groups(config)
 
         fd, tmp_file = tempfile.mkstemp()
-        f = open(tmp_file, 'w+')
-        formatter = generator._OptFormatter(output_file=f)
+        with open(tmp_file, 'w+') as f:
+            formatter = generator._OptFormatter(output_file=f)
+            generator._output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'))
         expected = '''[DEFAULT]
 '''
-        generator._output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'))
-        f.close()
-        content = open(tmp_file).read()
-        self.assertEqual(expected, content)
+        with open(tmp_file, 'r') as f:
+            actual = f.read()
+        self.assertEqual(expected, actual)
 
     def test_output_opts_group(self):
 
@@ -909,8 +987,9 @@ class GeneratorAdditionalTestCase(base.BaseTestCase):
         groups = generator._get_groups(config)
 
         fd, tmp_file = tempfile.mkstemp()
-        f = open(tmp_file, 'w+')
-        formatter = generator._OptFormatter(output_file=f)
+        with open(tmp_file, 'w+') as f:
+            formatter = generator._OptFormatter(output_file=f)
+            generator._output_opts(formatter, 'alpha', groups.pop('alpha'))
         expected = '''[alpha]
 
 #
@@ -920,10 +999,34 @@ class GeneratorAdditionalTestCase(base.BaseTestCase):
 # foo option (string value)
 #foo = fred
 '''
-        generator._output_opts(formatter, 'alpha', groups.pop('alpha'))
-        f.close()
-        content = open(tmp_file).read()
-        self.assertEqual(expected, content)
+        with open(tmp_file, 'r') as f:
+            actual = f.read()
+        self.assertEqual(expected, actual)
+
+
+class GeneratorMutableOptionTestCase(base.BaseTestCase):
+
+    def test_include_message(self):
+        out = moves.StringIO()
+        opt = cfg.StrOpt('foo', help='foo option', mutable=True)
+        gen = generator._OptFormatter(output_file=out)
+        gen.format(opt)
+        result = out.getvalue()
+        self.assertIn(
+            'This option can be changed without restarting.',
+            result,
+        )
+
+    def test_do_not_include_message(self):
+        out = moves.StringIO()
+        opt = cfg.StrOpt('foo', help='foo option', mutable=False)
+        gen = generator._OptFormatter(output_file=out)
+        gen.format(opt)
+        result = out.getvalue()
+        self.assertNotIn(
+            'This option can be changed without restarting.',
+            result,
+        )
 
 
 class GeneratorRaiseErrorTestCase(base.BaseTestCase):
